﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using System.Linq;

namespace Example_ConnectionComponent2
{
    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {
        /// <summary>
        /// 节点组件
        /// </summary>
        public ConnectionComponent TargetComp { get => ConnectComp; set => ConnectComp = value; }
        public ParamsHandler ParamsHandler { get; set; }
        public Beam MainPart { get; private set; }
        public BeamPlanes MainPlanes { get; private set; }
        public Beam SubPart { get; private set; }
        public BeamPlanes SubPlanes { get; private set; }
        public Vec3 CtrlPt { get; private set; }
        /// <summary>
        /// 公用ParamNames的数据
        /// </summary>
        public ConnectionComponent ConnectComp;


        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp, ParamsHandler handler)
        {
            var compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp, handler))
                return compContext;
            compContext.Clear();
            return null;
        }
        //具体的组件实例
        private bool Initialize(ConnectionComponent comp, ParamsHandler handler)
        {
            if (handler == null)
            {
                return false;
            }
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            Beam subPart1 = comp.Subs.Length > 0 ? comp.Subs.First<TFObject>() as Beam : null;
            if (mainPart == null || subPart1 == null)
                return false;
            TargetComp = comp;
            MainPart = mainPart;
            SubPart = subPart1;
            MainPlanes = BeamPlanes.CreateFromBeam(MainPart);
            SubPlanes = BeamPlanes.CreateFromBeam(SubPart);
            CtrlPt = TargetComp.WorldTagPos;
            return true;
        }
        private void Clear()
        {
            MainPart = null;
            SubPart = null;
            MainPlanes = null;
            SubPlanes = null;
        }
    }
}