﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using System.Windows.Forms;
namespace Example_ConnectionComponent2
{
    public class CreateStairController : ConnectionCompCreateController
    {
        public const string COMP_TAG_POS = "tag_pos";

        public CreateStairController(ComponentPlugin plugin) : base(plugin)
        {


        }

        private long m_IdComp0;
        private long m_IdComp1;

        int m_state;

        public override void OnActive(DObject paras)
        {
            base.OnActive(paras);
            CreateInfo.UserData = new DObject();
            SwitchStep0();
        }

        private void SwitchStep0()
        {
            BeginPick(Define.PICK_PART, Define.PICK_PART, Define.PICK_NONE);
            SetTip("创建组件：选择主零件");
            SetCursor(LoongCAD.STC.Define.MouseCursor.CURSOR_HAND);
            m_state = 0;
            m_IdComp0 = 0;
            m_IdComp1 = 0;
        }

        private void SwitchStep1(long id)
        {
            BeginPick(Define.PICK_PART, Define.PICK_PART, Define.PICK_NONE);
            SetTip("创建组件：选择次零件");
            SetCursor(LoongCAD.STC.Define.MouseCursor.CURSOR_HAND);
            m_IdComp0 = id;
            m_IdComp1 = 0;
            m_state = 1;
        }

        private void SwitchStep2(long id)
        {
            SetTip("创建组件：选择位置");
            SetCursor(LoongCAD.STC.Define.MouseCursor.CURSOR_CROSS);
            BeginSnapPoint();
            m_IdComp1 = id;
            m_state = 2;
        }

        public override void OnMouseClick(MouseButtons button, int x, int y, bool ctrl, bool alt, bool shift)
        {
            if (m_state == 0)
            {
                var ids = GetPickedObjects(ctrl, alt, shift);
                if (ids != null && ids.Length > 0)
                {
                    SwitchStep1(ids[0]);
                }
            }
            else if (m_state == 1)
            {
                var ids = GetPickedObjects(ctrl, alt, shift);
                if (ids != null && ids.Length > 0)
                {
                    SwitchStep2(ids[0]);
                }
            }
            base.OnMouseClick(button, x, y, ctrl, alt, shift);
        }

        public override bool IsObjectFocused(long id)
        {
            if (id == m_IdComp0 || id == m_IdComp1) { return true; }
            return false;
        }

        public override void OnDeactive()
        {
            m_state = 0;
            m_IdComp0 = 0;
            m_IdComp1 = 0;
            CreateInfo.UserData = null;
            base.OnDeactive();
        }

        public override void OnSnapPoint(Vec3 pt)
        {
            base.OnSnapPoint(pt);
            if (m_state == 2)
            {
                CreateInfo.MainId = m_IdComp0;
                CreateInfo.SubIds.Add(m_IdComp1);
                CreateInfo.UserData = new DObject();
                CreateInfo.UserData[COMP_TAG_POS] = pt;
                m_state = 0;
                CompleteCreation();
                Quit();
            }
        }

        public override void OnSnapRevert()
        {
            if (m_state == 1)
            {
                SwitchStep0();
            }
            else if (m_state == 2)
            {
                SwitchStep1(m_IdComp0);
            }
            base.OnSnapRevert();
        }

    }
}
