﻿using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.Util;
using System.Collections.Generic;
using System.Windows.Forms;
namespace Example_ConnectionComponent2
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private DObject m_weldProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private DObject m_rootUIProps = new DObject();

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            OnPostInitialize();
        }

        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        private void OnPostInitialize()
        {
            #region 额外的配置
            #endregion
            // 设置窗口标题
            Text = "自定义节点(001)";
            /// 若使用对象池，需要设置此属性为false
            DisposeOnClose = false;
            foreach (TabPage curPage in tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }

        }

        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_weldProps.Clear();
#if false
                m_weldProps.CopyFrom(paras);
#endif
            }
        }

        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck)
            {
                retProps.CopyFrom(m_weldProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }

        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            base.ClearUI(root);
        }


        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            SetPlaceholders(defaultValues);
        }


        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_weldProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = ReferencePlugin;
            if (refPlugin == null)
                return m_weldProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_weldProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_weldProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_weldProps;

            return comp.UIProps;
        }

        private void AddItemForKevValve(KeyValueComboBox ui, List<string> cfg)
        {
            ui.ClearItems();
            for (int i = 0; i < cfg.Count; i++)
            {
                ui.AddItem(i, cfg[i]);
            }
            ui.UpdateItems();
        }
    }
}
