﻿using LoongCAD.Util;
using System;
namespace Example_ConnectionComponent2
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {

        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数 
        /// </summary>
        public ParamsHandler()
        : base(EmptyValues, BuiltinDefaults) { }

        private static void AddEmptyDefaultValue(string paramName, object emptyValue, object defaultValue)
        {
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, paramName, emptyValue, defaultValue);
        }

        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        /// <remarks><b>每个组件需要分别实现此方法</b></remarks>
        private static void InitializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();
        }

        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null)
                {
                    InitializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }

        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject BuiltinDefaults
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    InitializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }

        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks> 
        public override DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefaults;
            if (emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine(@"ParamsHandler.InitializeDefaultValues:[Warning] EmptyValue/BuiltinDefault is null!");
                return m_defaultValues;
            }
            TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }

    }
}
