﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Example_ConnectionComponent2.Tool
{
    static class BoltsHelp
    {
        /// <summary>
        /// 处理间距
        /// </summary>
        /// <param name="array"></param>
        /// <param name="num"></param>
        /// <returns></returns>
        public static List<double> HandlerSpacing(double[] array, int num)
        {
            var ret = new List<double>();
            for (var i = 1; i < num; ++i)
            {
                ret.Add(array[Math.Min(i - 1, array.Length - 1)]);
            }
            return ret;

        }
        /// <summary>
        /// 自适应间距
        /// </summary>
        /// <param name="num"></param>
        /// <param name="space"></param>
        /// <param name="defaultSpace"></param>
        /// <returns></returns>
        public static double[] AdaptBoltSpacing(int num, double[] space, double defaultSpace)
        {
            if (num <= 0) return new double[] { };
            var ret = new List<double>();
            if (space.Length == 0)
            {
                ret.AddRange(Enumerable.Repeat(defaultSpace, num - 1));
                return ret.ToArray();
            }
            ret = HandlerSpacing(space, num);
            return ret.ToArray();
        }
    }

    public class BoltsData
    {
        public Mat43 m_workAxis;
        public double[] ArrayStrideX { get; set; }
        public double[] ArrayStrideY { get; set; }
        public double HeadOffsetX { get; set; }
        public double VerticalOffset { get; set; }
        //public double CutLength { get; set; }
        public Define.ObjectSide RotationSide { get; set; } = Define.ObjectSide.SIDE_TOP;
        /// <summary>
        /// 设置螺栓组
        /// </summary>
        /// <param name="boltGroup"></param>
        public void SetUpWith(BoltGroup boltGroup)
        {
            if (boltGroup == null)
                return;
            boltGroup.WorkAxis = m_workAxis;
            boltGroup.Rotation = 0.0d;
            boltGroup.RotationSide = RotationSide;
            //boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP;
            boltGroup.ArrayStrideX = ArrayStrideX;
            boltGroup.ArrayStrideY = ArrayStrideY;
            boltGroup.HeadOffsetX = HeadOffsetX;
            boltGroup.VerticalOffset = VerticalOffset;
            //boltGroup.DepthOffset = 9;
            //NOTE 每个支撑可能边缘厚度不一样，从而切割长度会不一样
            //boltGroup.CutLength = CutLength;
        }
    }
    public class BoltsConfig
    {

        /// <summary>
        /// 螺栓的基本属性值
        /// </summary>
        public class BoltsBasicProps
        {
            /// <summary>
            /// 螺栓标准 
            /// </summary>
            private string BoltStandard;
            /// <summary>
            /// 螺栓尺寸
            /// </summary>
            private double BoltSize;
            /// <summary>
            /// 螺栓孔容差
            /// </summary>
            private double HoleTol;
            /// <summary>
            /// 孔是否包含螺纹
            /// </summary>
            private bool ThreadInMaterial;
            /// <summary>
            /// 第一个工厂/工地属性
            /// </summary>
            public Define.PartMadeType MadeType0;
            /// <summary>
            /// 第二个工厂/工地属性
            /// </summary>
            public Define.PartMadeType MadeType1;
            /// <summary>
            /// 处理螺栓的基本属性值
            /// </summary>
            /// <param name="basicProps"></param>
            public void CopyFromBasicProps(DObject basicProps)
            {
                if (basicProps == null)
                    return;
                BoltStandard = basicProps.GetString(BoltBasicPropertyPanel.ParamNames.BoltStandard);
                BoltSize = basicProps.GetDouble(BoltBasicPropertyPanel.ParamNames.BoltSize);
                HoleTol = basicProps.GetDouble(BoltBasicPropertyPanel.ParamNames.HoleTol);
                ThreadInMaterial = basicProps.GetInt(BoltBasicPropertyPanel.ParamNames.ThreadInMtrl) == 2;
                MadeType0 = ParseMadeType(basicProps.GetInt(BoltBasicPropertyPanel.ParamNames.MadeType0));
                MadeType1 = ParseMadeType(basicProps.GetInt(BoltBasicPropertyPanel.ParamNames.MadeType1));
                if (!Equals(HoleTol, double.MaxValue)) return;
                if (BoltSize >= 27.0d)
                    HoleTol = 3.0d;
                else if (BoltSize >= 16.0d)
                    HoleTol = 2.0d;
                else
                    HoleTol = 1.5;
                return;
                //处理螺栓构建地点：工地、工厂
                Define.PartMadeType ParseMadeType(int comboIndex)
                {
                    switch (comboIndex)
                    {
                        case 2:
                            return Define.PartMadeType.PART_MADE_WORKSHOP;
                        default:
                            return Define.PartMadeType.PART_MADE_SITE;
                    }
                }
            }
            /// <summary>
            /// 填充给螺栓组
            /// </summary>
            /// <param name="boltGroup"></param>
            /// <param name="fillMadeType0">为真则用MadeType0进行填充</param>
            public void Fill(BoltGroup boltGroup, bool fillMadeType0 = true)
            {
                if (boltGroup == null)
                    return;
                boltGroup.BoltStandard = BoltStandard;
                boltGroup.BoltSize = BoltSize;
                boltGroup.HoleTol = HoleTol;
                boltGroup.ThreadInMtrl = ThreadInMaterial;
                boltGroup.MadeType = fillMadeType0 ? MadeType0 : MadeType1;
            }
        }

        /// <summary>
        /// 螺栓的孔相关配置
        /// </summary>
        private class BoltsHoleProps
        {
            private enum SlotInMode
            {
                None = 0, Beam = 1, Plate = 2, Both = 3,
            }
            private double HoleX;
            private double HoleY;
            private Define.BoltHoleType HoleType;
            private Define.BoltRotateSlot RotSlot;
            private SlotInMode SlotIn;
            /// <summary>
            /// 解析孔类型相关数据
            /// </summary>
            /// <param name="holeProps"></param>
            public void CopyFromHoleProps(DObject holeProps)
            {
                if (holeProps == null)
                    return;
                HoleX = holeProps.GetDouble(BoltHolePanel.ParamNames.HoleX);
                HoleY = holeProps.GetDouble(BoltHolePanel.ParamNames.HoleY);
                var comboIndex = holeProps.GetInt(BoltHolePanel.ParamNames.HoleType);
                switch (comboIndex)
                {
                    case 1:
                        HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
                        break;
                    default:
                        HoleType = Define.BoltHoleType.BOLT_HOLE_OVERSIZED;
                        break;
                }
                comboIndex = holeProps.GetInt(BoltHolePanel.ParamNames.RotSlot);
                switch (comboIndex)
                {
                    case 1:
                        RotSlot = Define.BoltRotateSlot.BOLT_SLOT_ODD;
                        break;
                    case 2:
                        RotSlot = Define.BoltRotateSlot.BOLT_SLOT_EVEN;
                        break;
                    default:
                        RotSlot = Define.BoltRotateSlot.BOLT_SLOT_PARALLEL;
                        break;
                }
                comboIndex = holeProps.GetInt(BoltHolePanel.ParamNames.SlotIn);
                switch (comboIndex)
                {
                    case 1:
                        SlotIn = SlotInMode.None;
                        break;
                    case 2:
                        SlotIn = SlotInMode.Beam;
                        break;
                    case 3:
                        SlotIn = SlotInMode.Plate;
                        break;
                    default:
                        SlotIn = SlotInMode.Both;
                        break;
                }
            }

            public void Fill(BoltGroup boltGroup)
            {
                if (boltGroup == null)
                    return;
                boltGroup.HoleX = HoleX;
                boltGroup.HoleY = HoleY;
                boltGroup.HoleType = HoleType;
                boltGroup.RotateSlot = RotSlot;
                //boltGroup
                if (SlotIn == SlotInMode.Beam)
                {
                    boltGroup.OverSizeHoleEnable0 = true;
                    boltGroup.OverSizeHoleEnable1 = false;
                    boltGroup.OverSizeHoleEnable2 = false;
                    boltGroup.OverSizeHoleEnable3 = false;
                    boltGroup.OverSizeHoleEnable4 = false;
                }
                else if (SlotIn == SlotInMode.Plate)
                {
                    boltGroup.OverSizeHoleEnable0 = false;
                    boltGroup.OverSizeHoleEnable1 = true;
                    boltGroup.OverSizeHoleEnable2 = true;
                    boltGroup.OverSizeHoleEnable3 = false;
                    boltGroup.OverSizeHoleEnable4 = false;

                }
                else if (SlotIn == SlotInMode.Both)
                {
                    boltGroup.OverSizeHoleEnable0 = true;
                    boltGroup.OverSizeHoleEnable1 = true;
                    boltGroup.OverSizeHoleEnable2 = true;
                    boltGroup.OverSizeHoleEnable3 = true;
                    boltGroup.OverSizeHoleEnable4 = true;

                }
                else if (SlotIn == SlotInMode.None)
                {
                    boltGroup.OverSizeHoleEnable0 = false;
                    boltGroup.OverSizeHoleEnable1 = false;
                    boltGroup.OverSizeHoleEnable2 = false;
                    boltGroup.OverSizeHoleEnable3 = false;
                    boltGroup.OverSizeHoleEnable4 = false;

                }
            }
        }

        /// <summary>
        /// 螺栓组件的启用配置
        /// </summary>
        private class BoltsContentProps
        {
            private bool PartWasherUp;
            private bool PartWasherDown0;
            private bool PartWasherDown1;
            private bool PartNut0;
            private bool PartNut1;
            public void CopyFromContentProps(DObject contentProps)
            {
                if (contentProps == null)
                    return;
                PartWasherUp = contentProps.GetBool(BoltContentPanel.ParamNames.PartWasherUp);
                PartWasherDown0 = contentProps.GetBool(BoltContentPanel.ParamNames.PartWasherDown0);
                PartWasherDown1 = contentProps.GetBool(BoltContentPanel.ParamNames.PartWasherDown1);
                PartNut0 = contentProps.GetBool(BoltContentPanel.ParamNames.PartNut0);
                PartNut1 = contentProps.GetBool(BoltContentPanel.ParamNames.PartNut1);
            }

            public void Fill(BoltGroup boltGroup)
            {
                if (boltGroup == null)
                    return;
                boltGroup.WasherUpEnable = PartWasherUp;
                boltGroup.WasherDownEnable0 = PartWasherDown0;
                boltGroup.WasherDownEnable1 = PartWasherDown1;
                boltGroup.NutEnable0 = PartNut0;
                boltGroup.NutEnable1 = PartNut1;
            }
        }

        /// <summary>
        /// 螺栓的基本属性值
        /// </summary>
        public BoltsBasicProps m_basicProps = new BoltsBasicProps();
        /// <summary>
        /// 螺栓的孔相关配置
        /// </summary>
        private BoltsHoleProps m_holeProps = new BoltsHoleProps();
        /// <summary>
        /// 螺栓组件的启用配置
        /// </summary>
        private BoltsContentProps m_contentProps = new BoltsContentProps();
        /// <summary>
        /// 螺栓的其他属性值
        /// </summary>
        public double m_extendLength;
        public Define.BoltGroupShape m_shape;


        /// <summary>
        /// 处理螺栓的基本属性值
        /// </summary>
        /// <param name="basicProps"></param>
        public void CopyFromBasicProps(DObject basicProps)
        {
            m_basicProps?.CopyFromBasicProps(basicProps);
        }
        /// <summary>
        /// 解析孔类型、槽孔相关配置
        /// </summary>
        /// <param name="holeProps"></param>
        public void CopyFromHoleProps(DObject holeProps)
        {
            m_holeProps.CopyFromHoleProps(holeProps);
        }
        /// <summary>
        /// 解析螺栓组件的装配相关配置
        /// </summary>
        /// <param name="contentProps"></param>
        public void CopyFromContentProps(DObject contentProps)
        {
            m_contentProps.CopyFromContentProps(contentProps);
        }
        /// <summary>
        /// 解析螺栓的附加长度配置
        /// </summary>
        /// <param name="extendProps"></param>
        public void CopyFromExtendProps(DObject extendProps)
        {
            if (extendProps == null)
                return;
            m_extendLength = extendProps.GetDouble(BoltExtendPanel.ParamNames.ExtLength);
        }

        /// <summary>
        /// 解析螺栓的排列方式配置
        /// </summary>
        /// <param name="shapeProps"></param>
        public void CopyFromShapeProps(int shapeProps)
        {
            switch (shapeProps)
            {
                case 2:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_ODD;
                    break;
                case 3:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_EVEN;
                    break;
                case 4:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_ODD;
                    break;
                case 5:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_EVEN;
                    break;
                default:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ARRAY;
                    break;
            }
        }

        /// <summary>
        /// 填充螺栓组的螺栓基本属性
        /// </summary>
        /// <param name="boltGroup"></param>
        /// <param name="fillMadeType0"></param>
        public void FillBasicProps(BoltGroup boltGroup, bool fillMadeType0 = true)
        {
            m_basicProps?.Fill(boltGroup, fillMadeType0);
        }
        /// <summary>
        /// 填充螺栓组的螺栓形状
        /// </summary>
        /// <param name="boltGroup"></param>
        public void FillShape(BoltGroup boltGroup)
        {
            if (boltGroup == null)
                return;
            boltGroup.Shape = m_shape;

        }
        /// <summary>
        /// 填充螺栓组的孔配置
        /// </summary>
        /// <param name="boltGroup"></param>
        public void FillHoleProps(BoltGroup boltGroup)
        {
            m_holeProps?.Fill(boltGroup);
        }
        /// <summary>
        /// 填充螺栓组的启用配置
        /// </summary>
        /// <param name="boltGroup"></param>
        public void FillContentProps(BoltGroup boltGroup)
        {
            m_contentProps?.Fill(boltGroup);
        }
        /// <summary>
        /// 填充螺栓组的附加长度
        /// </summary>
        /// <param name="boltGroup"></param>
        public void FillExtendProps(BoltGroup boltGroup)
        {
            if (boltGroup != null)
            {
                boltGroup.ExtLength = m_extendLength;
            }
        }

        /// <summary>
        /// 执行所有的填充函数
        /// </summary>
        /// <param name="boltGroup"></param>
        public void FillAll(BoltGroup boltGroup)
        {
            FillShape(boltGroup);
            FillBasicProps(boltGroup);
            FillContentProps(boltGroup);
            FillExtendProps(boltGroup);
            FillHoleProps(boltGroup);
        }
    }
}