﻿using System;
using System.Diagnostics;
using System.IO;

namespace Example_ConnectionComponent2.Tool
{
    public sealed class Log : IDisposable
    {
        private static readonly Lazy<Log> lazy = new Lazy<Log>(() => new Log());
        public static Log Instance { get { return lazy.Value; } }

        private StreamWriter logFile;
        private int currentLine = 0;
        private const int maxLines = 1000;
        private Process powershellProcess;

        private Log()
        {
            logFile = new StreamWriter("log.txt", true);
            OpenPowerShell();
        }

        public void Dispose()
        {
            logFile?.Dispose();
            if (powershellProcess != null && !powershellProcess.HasExited)
            {
                powershellProcess.Kill();
            }
        }

        private void WriteToFile(string message)
        {
            if (logFile != null)
            {
                if (currentLine >= maxLines)
                {
                    logFile.BaseStream.Seek(0, SeekOrigin.Begin);
                    currentLine = 0;
                }
                logFile.WriteLine(message);
                logFile.Flush();
                currentLine++;
            }
            else
            {
                Console.Error.WriteLine("Unable to write to log file.");
            }
        }

        public void LogError(string message)
        {
            string fullMessage = $"[ERROR] {currentLine.ToString(),4}|{DateTime.Now}: {message} ";
            WriteToFile(fullMessage);
        }

        public void LogInfo(string message)
        {
            string fullMessage = $"[INFO] {currentLine.ToString(),4}|{DateTime.Now}: {message} ";
            WriteToFile(fullMessage);
        }

        private void OpenPowerShell()
        {
            string command = "-NoExit -Command \"Get-Content log.txt -Wait\"";
            powershellProcess = new Process
            {
                StartInfo = new ProcessStartInfo
                {
                    FileName = "powershell.exe",
                    Arguments = command,
                    RedirectStandardOutput = false,
                    UseShellExecute = true,
                    CreateNoWindow = false
                }
            };
            powershellProcess.Start();
        }
    }
}
